@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,23 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
+/* This file contains the code necessary for generating on canvas
+ * previews, either by connecting a function to process the pixels or
+ * by connecting a specified GEGL operation to do the processing. It
+ * keeps an undo buffer to allow direct modification of the pixel data
+ * (so that it will show up in the projection) and it will restore the
+ * source in case the mapping procedure was cancelled.
+ *
+ * To create a tool that uses this, see /tools/gimpimagemaptool.c for
+ * the interface and /tools/gimpcolorbalancetool.c for an example of
+ * using that interface.
+ *
+ * Note that when talking about on canvas preview, we are speaking
+ * about non destructive image editing where the operation is previewd
+ * before being applied.
  */
 
 #include "config.h"
@@ -29,8 +44,6 @@
 
 #include "paint-funcs/paint-funcs.h"
 
-#include "gegl/gimp-gegl-utils.h"
-
 #include "gimpdrawable.h"
 #include "gimpdrawable-shadow.h"
 #include "gimpimage.h"
@@ -39,16 +52,6 @@
 #include "gimppickable.h"
 #include "gimpviewable.h"
 
-#ifdef __GNUC__
-#warning FIXME: gegl_node_add_child() needs to be public
-#endif
-GeglNode * gegl_node_add_child (GeglNode *self,
-                                GeglNode *child);
-#ifdef __GNUC__
-#warning FIXME: gegl_node_get_pad() or something similar needs to be public
-#endif
-gpointer   gegl_node_get_pad (GeglNode    *self,
-                              const gchar *pad_name);
 
 enum
 {
@@ -76,7 +79,7 @@ struct _GimpImageMap
 
   GeglNode              *gegl;
   GeglNode              *input;
-  GeglNode              *shift;
+  GeglNode              *translate;
   GeglNode              *operation;
   GeglNode              *output;
   GeglProcessor         *processor;
@@ -102,10 +105,17 @@ static gboolean        gimp_image_map_get_pixel_at   (GimpPickable        *picka
                                                       gint                 y,
                                                       guchar              *pixel);
 
+static void            gimp_image_map_update_undo_tiles
+                                                     (GimpImageMap        *image_map,
+                                                      const GeglRectangle *rect);
 static gboolean        gimp_image_map_do             (GimpImageMap        *image_map);
 static void            gimp_image_map_data_written   (GObject             *operation,
                                                       const GeglRectangle *extent,
                                                       GimpImageMap        *image_map);
+static void            gimp_image_map_cancel_any_idle_jobs
+                                                     (GimpImageMap        *image_map);
+static void            gimp_image_map_kill_any_idle_processors
+                                                     (GimpImageMap        *image_map);
 
 
 G_DEFINE_TYPE_WITH_CODE (GimpImageMap, gimp_image_map, GIMP_TYPE_OBJECT,
@@ -198,31 +208,15 @@ gimp_image_map_finalize (GObject *object)
       image_map->undo_tiles = NULL;
     }
 
-  if (image_map->idle_id)
-    {
-      g_source_remove (image_map->idle_id);
-      image_map->idle_id = 0;
-
-      if (image_map->processor)
-        {
-          g_object_unref (image_map->processor);
-          image_map->processor = NULL;
-        }
-
-      if (image_map->PRI)
-        {
-          pixel_regions_process_stop (image_map->PRI);
-          image_map->PRI = NULL;
-        }
-    }
+  gimp_image_map_cancel_any_idle_jobs (image_map);
 
   if (image_map->gegl)
     {
       g_object_unref (image_map->gegl);
-      image_map->gegl = NULL;
-      image_map->input = NULL;
-      image_map->shift = NULL;
-      image_map->output = NULL;
+      image_map->gegl      = NULL;
+      image_map->input     = NULL;
+      image_map->translate = NULL;
+      image_map->output    = NULL;
     }
 
   if (image_map->operation)
@@ -292,8 +286,8 @@ gimp_image_map_get_pixel_at (GimpPickable *pickable,
   GimpImageMap *image_map = GIMP_IMAGE_MAP (pickable);
   GimpItem     *item      = GIMP_ITEM (image_map->drawable);
 
-  if (x >= 0 && x < gimp_item_width  (item) &&
-      y >= 0 && y < gimp_item_height (item))
+  if (x >= 0 && x < gimp_item_get_width  (item) &&
+      y >= 0 && y < gimp_item_get_height (item))
     {
       /* Check if done damage to original image */
       if (image_map->undo_tiles)
@@ -306,10 +300,10 @@ gimp_image_map_get_pixel_at (GimpPickable *pickable,
           if (x >= offset_x && x < offset_x + width &&
               y >= offset_y && y < offset_y + height)
             {
-              read_pixel_data_1 (image_map->undo_tiles,
-                                 x - offset_x,
-                                 y - offset_y,
-                                 pixel);
+              tile_manager_read_pixel_data_1 (image_map->undo_tiles,
+                                              x - offset_x,
+                                              y - offset_y,
+                                              pixel);
 
               return TRUE;
             }
@@ -359,130 +353,51 @@ gimp_image_map_apply (GimpImageMap        *image_map,
                       const GeglRectangle *visible)
 {
   GeglRectangle rect;
-  gint          undo_offset_x;
-  gint          undo_offset_y;
-  gint          undo_width;
-  gint          undo_height;
 
   g_return_if_fail (GIMP_IS_IMAGE_MAP (image_map));
 
   /*  If we're still working, remove the timer  */
-  if (image_map->idle_id)
-    {
-      g_source_remove (image_map->idle_id);
-      image_map->idle_id = 0;
-
-      if (image_map->processor)
-        {
-          g_object_unref (image_map->processor);
-          image_map->processor = NULL;
-        }
-
-      if (image_map->PRI)
-        {
-          pixel_regions_process_stop (image_map->PRI);
-          image_map->PRI = NULL;
-        }
-    }
+  gimp_image_map_cancel_any_idle_jobs (image_map);
 
   /*  Make sure the drawable is still valid  */
   if (! gimp_item_is_attached (GIMP_ITEM (image_map->drawable)))
     return;
 
   /*  The application should occur only within selection bounds  */
-  if (! gimp_drawable_mask_intersect (image_map->drawable,
-                                      &rect.x, &rect.y,
-                                      &rect.width, &rect.height))
+  if (! gimp_item_mask_intersect (GIMP_ITEM (image_map->drawable),
+                                  &rect.x, &rect.y,
+                                  &rect.width, &rect.height))
     return;
 
   /*  If undo tiles don't exist, or change size, (re)allocate  */
-  if (image_map->undo_tiles)
-    {
-      undo_offset_x = image_map->undo_offset_x;
-      undo_offset_y = image_map->undo_offset_y;
-      undo_width    = tile_manager_width  (image_map->undo_tiles);
-      undo_height   = tile_manager_height (image_map->undo_tiles);
-    }
-  else
-    {
-      undo_offset_x = 0;
-      undo_offset_y = 0;
-      undo_width    = 0;
-      undo_height   = 0;
-    }
-
-  if (! image_map->undo_tiles     ||
-      undo_offset_x != rect.x     ||
-      undo_offset_y != rect.y     ||
-      undo_width    != rect.width ||
-      undo_height   != rect.height)
-    {
-      /* If either the extents changed or the tiles don't exist,
-       * allocate new
-       */
-      if (! image_map->undo_tiles   ||
-          undo_width  != rect.width ||
-          undo_height != rect.height)
-        {
-          /*  Destroy old tiles  */
-          if (image_map->undo_tiles)
-            tile_manager_unref (image_map->undo_tiles);
-
-          /*  Allocate new tiles  */
-          image_map->undo_tiles =
-            tile_manager_new (rect.width, rect.height,
-                              gimp_drawable_bytes (image_map->drawable));
-        }
-
-      /*  Copy from the image to the new tiles  */
-      pixel_region_init (&image_map->srcPR,
-                         gimp_drawable_get_tiles (image_map->drawable),
-                         rect.x, rect.y,
-                         rect.width, rect.height,
-                         FALSE);
-      pixel_region_init (&image_map->destPR,
-                         image_map->undo_tiles,
-                         0, 0,
-                         rect.width, rect.height,
-                         TRUE);
-
-      copy_region (&image_map->srcPR, &image_map->destPR);
-
-      /*  Set the offsets  */
-      image_map->undo_offset_x = rect.x;
-      image_map->undo_offset_y = rect.y;
-    }
+  gimp_image_map_update_undo_tiles (image_map,
+                                    &rect);
 
   if (image_map->operation)
     {
       if (! image_map->gegl)
         {
-	  const gchar *shift_name;
-
           image_map->gegl = gegl_node_new ();
 
-          if (g_object_class_find_property (
-              G_OBJECT_GET_CLASS (image_map->gegl), "dont-cache"))
-            g_object_set (image_map->gegl, "dont-cache", TRUE, NULL);
+          g_object_set (image_map->gegl,
+                        "dont-cache", TRUE,
+                        NULL);
 
           image_map->input =
             gegl_node_new_child (image_map->gegl,
-                                 "operation", "gimp-tilemanager-source",
+                                 "operation", "gimp:tilemanager-source",
                                  NULL);
 
-	  shift_name =
-	    gimp_gegl_check_version (0, 0, 21) ? "gegl:translate" : "shift";
-
-          image_map->shift =
+          image_map->translate =
             gegl_node_new_child (image_map->gegl,
-                                 "operation", shift_name,
+                                 "operation", "gegl:translate",
                                  NULL);
 
           gegl_node_add_child (image_map->gegl, image_map->operation);
 
           image_map->output =
             gegl_node_new_child (image_map->gegl,
-                                 "operation", "gimp-tilemanager-sink",
+                                 "operation", "gimp:tilemanager-sink",
                                  NULL);
 
           {
@@ -499,36 +414,30 @@ gimp_image_map_apply (GimpImageMap        *image_map,
             g_object_unref (sink_operation);
           }
 
-          if (gegl_node_get_pad (image_map->operation, "input") &&
-              gegl_node_get_pad (image_map->operation, "output"))
+          if (gegl_node_has_pad (image_map->operation, "input") &&
+              gegl_node_has_pad (image_map->operation, "output"))
             {
               /*  if there are input and output pads we probably have a
                *  filter OP, connect it on both ends.
                */
               gegl_node_link_many (image_map->input,
-                                   image_map->shift,
+                                   image_map->translate,
                                    image_map->operation,
                                    image_map->output,
                                    NULL);
             }
-          else if (gegl_node_get_pad (image_map->operation, "output"))
+          else if (gegl_node_has_pad (image_map->operation, "output"))
             {
               /*  if there is only an output pad we probably have a
                *  source OP, blend its result on top of the original
                *  pixels.
                */
-	      const gchar *over_name;
-              GeglNode    *over;
-
-	      over_name =
-		gimp_gegl_check_version (0, 0, 21) ? "gegl:over" : "over";
-
-	      over = gegl_node_new_child (image_map->gegl,
-					  "operation", over_name,
-					  NULL);
+              GeglNode *over = gegl_node_new_child (image_map->gegl,
+                                                    "operation", "gegl:over",
+                                                    NULL);
 
               gegl_node_link_many (image_map->input,
-                                   image_map->shift,
+                                   image_map->translate,
                                    over,
                                    image_map->output,
                                    NULL);
@@ -541,7 +450,7 @@ gimp_image_map_apply (GimpImageMap        *image_map,
               /* otherwise we just construct a silly nop pipleline
                */
               gegl_node_link_many (image_map->input,
-                                   image_map->shift,
+                                   image_map->translate,
                                    image_map->output,
                                    NULL);
             }
@@ -552,7 +461,7 @@ gimp_image_map_apply (GimpImageMap        *image_map,
                      "linear",       TRUE,
                      NULL);
 
-      gegl_node_set (image_map->shift,
+      gegl_node_set (image_map->translate,
                      "x", (gdouble) rect.x,
                      "y", (gdouble) rect.y,
                      NULL);
@@ -601,8 +510,6 @@ gimp_image_map_apply (GimpImageMap        *image_map,
 void
 gimp_image_map_commit (GimpImageMap *image_map)
 {
-  gint x1, y1, x2, y2;
-
   g_return_if_fail (GIMP_IS_IMAGE_MAP (image_map));
 
   if (image_map->idle_id)
@@ -621,14 +528,14 @@ gimp_image_map_commit (GimpImageMap *image_map)
   /*  Register an undo step  */
   if (image_map->undo_tiles)
     {
-      x1 = image_map->undo_offset_x;
-      y1 = image_map->undo_offset_y;
-      x2 = x1 + tile_manager_width (image_map->undo_tiles);
-      y2 = y1 + tile_manager_height (image_map->undo_tiles);
+      gint x      = image_map->undo_offset_x;
+      gint y      = image_map->undo_offset_y;
+      gint width  = tile_manager_width  (image_map->undo_tiles);
+      gint height = tile_manager_height (image_map->undo_tiles);
 
       gimp_drawable_push_undo (image_map->drawable,
                                image_map->undo_desc,
-                               x1, y1, x2, y2,
+                               x, y, width, height,
                                image_map->undo_tiles, FALSE);
 
       tile_manager_unref (image_map->undo_tiles);
@@ -641,23 +548,7 @@ gimp_image_map_clear (GimpImageMap *image_map)
 {
   g_return_if_fail (GIMP_IS_IMAGE_MAP (image_map));
 
-  if (image_map->idle_id)
-    {
-      g_source_remove (image_map->idle_id);
-      image_map->idle_id = 0;
-
-      if (image_map->processor)
-        {
-          g_object_unref (image_map->processor);
-          image_map->processor = NULL;
-        }
-
-      if (image_map->PRI)
-        {
-          pixel_regions_process_stop (image_map->PRI);
-          image_map->PRI = NULL;
-        }
-    }
+  gimp_image_map_cancel_any_idle_jobs (image_map);
 
   /*  Make sure the drawable is still valid  */
   if (! gimp_item_is_attached (GIMP_ITEM (image_map->drawable)))
@@ -704,23 +595,7 @@ gimp_image_map_abort (GimpImageMap *image_map)
 {
   g_return_if_fail (GIMP_IS_IMAGE_MAP (image_map));
 
-  if (image_map->idle_id)
-    {
-      g_source_remove (image_map->idle_id);
-      image_map->idle_id = 0;
-
-      if (image_map->processor)
-        {
-          g_object_unref (image_map->processor);
-          image_map->processor = NULL;
-        }
-
-      if (image_map->PRI)
-        {
-          pixel_regions_process_stop (image_map->PRI);
-          image_map->PRI = NULL;
-        }
-    }
+  gimp_image_map_cancel_any_idle_jobs (image_map);
 
   if (! gimp_item_is_attached (GIMP_ITEM (image_map->drawable)))
     return;
@@ -731,6 +606,73 @@ gimp_image_map_abort (GimpImageMap *image_map)
 
 /*  private functions  */
 
+static void
+gimp_image_map_update_undo_tiles (GimpImageMap        *image_map,
+                                  const GeglRectangle *rect)
+{
+  gint undo_offset_x;
+  gint undo_offset_y;
+  gint undo_width;
+  gint undo_height;
+
+  if (image_map->undo_tiles)
+    {
+      undo_offset_x = image_map->undo_offset_x;
+      undo_offset_y = image_map->undo_offset_y;
+      undo_width    = tile_manager_width  (image_map->undo_tiles);
+      undo_height   = tile_manager_height (image_map->undo_tiles);
+    }
+  else
+    {
+      undo_offset_x = 0;
+      undo_offset_y = 0;
+      undo_width    = 0;
+      undo_height   = 0;
+    }
+
+  if (! image_map->undo_tiles     ||
+      undo_offset_x != rect->x     ||
+      undo_offset_y != rect->y     ||
+      undo_width    != rect->width ||
+      undo_height   != rect->height)
+    {
+      /* If either the extents changed or the tiles don't exist,
+       * allocate new
+       */
+      if (! image_map->undo_tiles   ||
+          undo_width  != rect->width ||
+          undo_height != rect->height)
+        {
+          /*  Destroy old tiles  */
+          if (image_map->undo_tiles)
+            tile_manager_unref (image_map->undo_tiles);
+
+          /*  Allocate new tiles  */
+          image_map->undo_tiles =
+            tile_manager_new (rect->width, rect->height,
+                              gimp_drawable_bytes (image_map->drawable));
+        }
+
+      /*  Copy from the image to the new tiles  */
+      pixel_region_init (&image_map->srcPR,
+                         gimp_drawable_get_tiles (image_map->drawable),
+                         rect->x, rect->y,
+                         rect->width, rect->height,
+                         FALSE);
+      pixel_region_init (&image_map->destPR,
+                         image_map->undo_tiles,
+                         0, 0,
+                         rect->width, rect->height,
+                         TRUE);
+
+      copy_region (&image_map->srcPR, &image_map->destPR);
+
+      /*  Set the offsets  */
+      image_map->undo_offset_x = rect->x;
+      image_map->undo_offset_y = rect->y;
+    }
+}
+
 static gboolean
 gimp_image_map_do (GimpImageMap *image_map)
 {
@@ -738,19 +680,9 @@ gimp_image_map_do (GimpImageMap *image_map)
     {
       image_map->idle_id = 0;
 
-      if (image_map->processor)
-        {
-          g_object_unref (image_map->processor);
-          image_map->processor = NULL;
-        }
+      gimp_image_map_kill_any_idle_processors (image_map);
 
-      if (image_map->PRI)
-        {
-          pixel_regions_process_stop (image_map->PRI);
-          image_map->PRI = NULL;
-        }
-
-     return FALSE;
+      return FALSE;
     }
 
   if (image_map->gegl)
@@ -828,7 +760,7 @@ gimp_image_map_do (GimpImageMap *image_map)
           gimp_drawable_apply_region (image_map->drawable, &srcPR,
                                       FALSE, NULL,
                                       GIMP_OPACITY_OPAQUE, GIMP_REPLACE_MODE,
-                                      NULL,
+                                      NULL, NULL,
                                       x, y);
 
           gimp_drawable_update (image_map->drawable, x, y, w, h);
@@ -904,7 +836,7 @@ gimp_image_map_data_written (GObject             *operation,
   gimp_drawable_apply_region (image_map->drawable, &srcPR,
                               FALSE, NULL,
                               GIMP_OPACITY_OPAQUE, GIMP_REPLACE_MODE,
-                              NULL,
+                              NULL, NULL,
                               extent->x, extent->y);
 
   gimp_drawable_update (image_map->drawable,
@@ -914,3 +846,31 @@ gimp_image_map_data_written (GObject             *operation,
   if (image_map->timer)
     image_map->pixel_count += extent->width * extent->height;
 }
+
+static void
+gimp_image_map_cancel_any_idle_jobs (GimpImageMap *image_map)
+{
+  if (image_map->idle_id)
+    {
+      g_source_remove (image_map->idle_id);
+      image_map->idle_id = 0;
+
+      gimp_image_map_kill_any_idle_processors (image_map);
+    }
+}
+
+static void
+gimp_image_map_kill_any_idle_processors (GimpImageMap *image_map)
+{
+  if (image_map->processor)
+    {
+      g_object_unref (image_map->processor);
+      image_map->processor = NULL;
+    }
+
+  if (image_map->PRI)
+    {
+      pixel_regions_process_stop (image_map->PRI);
+      image_map->PRI = NULL;
+    }
+}
