@@ -26,6 +26,7 @@
 
 #include "text-types.h"
 
+#include "core/gimp.h"
 #include "core/gimpimage.h"
 
 #include "vectors/gimpbezierstroke.h"
@@ -69,24 +70,24 @@ gimp_text_vectors_new (GimpImage *image,
       cairo_t         *cr;
       gdouble          xres;
       gdouble          yres;
+      GError          *error = NULL;
 
       if (text->text)
         gimp_object_set_name_safe (GIMP_OBJECT (vectors), text->text);
 
       context.vectors = vectors;
 
-      /* A cairo_t needs an image surface to function, so "surface" is
-       * created temporarily for this purpose. Nothing is drawn to
-       * "surface", but it is still needed to be connected to "cr" for
-       * "cr" to execute cr_glyph_path(). The size of surface is
-       * therefore irrelevant.
-       */
-      surface = cairo_image_surface_create (CAIRO_FORMAT_A8, 2, 2);
+      surface = cairo_recording_surface_create (CAIRO_CONTENT_ALPHA, NULL);
       cr = cairo_create (surface);
 
       gimp_image_get_resolution (image, &xres, &yres);
 
-      layout = gimp_text_layout_new (text, xres, yres);
+      layout = gimp_text_layout_new (text, xres, yres, &error);
+      if (error)
+        {
+          gimp_message_literal (image->gimp, NULL, GIMP_MESSAGE_ERROR, error->message);
+          g_error_free (error);
+        }
       gimp_text_layout_render (layout, cr, text->base_dir, TRUE);
       g_object_unref (layout);
 
