@@ -4,10 +4,10 @@
  * gimpcolorarea.c
  * Copyright (C) 2001-2002  Sven Neumann <sven@gimp.org>
  *
- * This library is free software; you can redistribute it and/or
+ * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,9 +15,8 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -29,9 +28,20 @@
 
 #include "gimpwidgetstypes.h"
 
+#include "gimpcairo-utils.h"
 #include "gimpcolorarea.h"
 
 
+/**
+ * SECTION: gimpcolorarea
+ * @title: GimpColorArea
+ * @short_description: Displays a #GimpRGB color, optionally with
+ *                     alpha-channel.
+ *
+ * Displays a #GimpRGB color, optionally with alpha-channel.
+ **/
+
+
 #define DRAG_PREVIEW_SIZE   32
 #define DRAG_ICON_OFFSET    -8
 
@@ -68,6 +78,14 @@ static void      gimp_color_area_state_changed (GtkWidget          *widget,
                                                 GtkStateType        previous_state);
 static gboolean  gimp_color_area_expose        (GtkWidget          *widget,
                                                 GdkEventExpose     *event);
+static void      gimp_color_area_render_buf    (GtkWidget          *widget,
+                                                gboolean            insensitive,
+                                                GimpColorAreaType   type,
+                                                guchar             *buf,
+                                                guint               width,
+                                                guint               height,
+                                                guint               rowstride,
+                                                GimpRGB            *color);
 static void      gimp_color_area_render        (GimpColorArea      *area);
 
 static void  gimp_color_area_drag_begin         (GtkWidget        *widget,
@@ -290,13 +308,13 @@ gimp_color_area_size_allocate (GtkWidget     *widget,
   if (GTK_WIDGET_CLASS (parent_class)->size_allocate)
     GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
 
-  if (widget->allocation.width  != area->width ||
-      widget->allocation.height != area->height)
+  if (allocation->width  != area->width ||
+      allocation->height != area->height)
     {
-      area->width  = widget->allocation.width;
-      area->height = widget->allocation.height;
+      area->width  = allocation->width;
+      area->height = allocation->height;
 
-      area->rowstride = (area->width * 3 + 3) & ~0x3;
+      area->rowstride = area->width * 4 + 4;
 
       g_free (area->buf);
       area->buf = g_new (guchar, area->rowstride * area->height);
@@ -309,7 +327,7 @@ static void
 gimp_color_area_state_changed (GtkWidget    *widget,
                                GtkStateType  previous_state)
 {
-  if (widget->state == GTK_STATE_INSENSITIVE ||
+  if (gtk_widget_get_state (widget) == GTK_STATE_INSENSITIVE ||
       previous_state == GTK_STATE_INSENSITIVE)
     {
       GIMP_COLOR_AREA (widget)->needs_render = TRUE;
@@ -323,36 +341,44 @@ static gboolean
 gimp_color_area_expose (GtkWidget      *widget,
                         GdkEventExpose *event)
 {
-  GimpColorArea *area  = GIMP_COLOR_AREA (widget);
-  GtkStyle      *style = gtk_widget_get_style (widget);
-  guchar        *buf;
+  GimpColorArea   *area  = GIMP_COLOR_AREA (widget);
+  GtkStyle        *style = gtk_widget_get_style (widget);
+  cairo_t         *cr;
+  cairo_surface_t *buffer;
 
-  if (! area->buf || ! GTK_WIDGET_DRAWABLE (widget))
+  if (! area->buf || ! gtk_widget_is_drawable (widget))
     return FALSE;
 
   if (area->needs_render)
     gimp_color_area_render (area);
 
-  buf = area->buf + event->area.y * area->rowstride + event->area.x * 3;
+  cr = gdk_cairo_create (gtk_widget_get_window (widget));
 
-  gdk_draw_rgb_image_dithalign (widget->window,
-                                style->black_gc,
-                                event->area.x,
-                                event->area.y,
-                                event->area.width,
-                                event->area.height,
-                                GDK_RGB_DITHER_MAX,
-                                buf,
-                                area->rowstride,
-                                event->area.x,
-                                event->area.y);
+  gdk_cairo_region (cr, event->region);
+  cairo_clip (cr);
+
+  buffer = cairo_image_surface_create_for_data (area->buf,
+                                                CAIRO_FORMAT_RGB24,
+                                                area->width,
+                                                area->height,
+                                                area->rowstride);
+  cairo_set_source_surface (cr, buffer, 0.0, 0.0);
+  cairo_surface_destroy (buffer);
+
+  cairo_paint (cr);
 
   if (area->draw_border)
-    gdk_draw_rectangle (widget->window,
-                        style->fg_gc[widget->state],
-                        FALSE,
-                        0, 0,
-                        area->width - 1, area->height - 1);
+    {
+      cairo_set_line_width (cr, 1.0);
+      gdk_cairo_set_source_color (cr,
+                                  &style->fg[gtk_widget_get_state (widget)]);
+
+      cairo_rectangle (cr, 0.5, 0.5, area->width - 1, area->height - 1);
+
+      cairo_stroke (cr);
+    }
+
+  cairo_destroy (cr);
 
   return FALSE;
 }
@@ -496,17 +522,17 @@ gimp_color_area_set_draw_border (GimpColorArea *area,
     }
 }
 
-void
-_gimp_color_area_render_buf (GtkWidget         *widget,
-                             gboolean           insensitive,
-                             GimpColorAreaType  type,
-                             guchar            *buf,
-                             guint              width,
-                             guint              height,
-                             guint              rowstride,
-                             GimpRGB           *color)
+static void
+gimp_color_area_render_buf (GtkWidget         *widget,
+                            gboolean           insensitive,
+                            GimpColorAreaType  type,
+                            guchar            *buf,
+                            guint              width,
+                            guint              height,
+                            guint              rowstride,
+                            GimpRGB           *color)
 {
-  GtkStyle *style;
+  GtkStyle *style = gtk_widget_get_style (widget);
   guint     x, y;
   guint     check_size = 0;
   guchar    light[3];
@@ -516,12 +542,6 @@ _gimp_color_area_render_buf (GtkWidget         *widget,
   guchar   *p;
   gdouble   frac;
 
-  g_return_if_fail (GTK_IS_WIDGET (widget));
-  g_return_if_fail (buf != NULL);
-  g_return_if_fail (color != NULL);
-
-  style = gtk_widget_get_style (widget);
-
   switch (type)
     {
     case GIMP_COLOR_AREA_FLAT:
@@ -553,16 +573,20 @@ _gimp_color_area_render_buf (GtkWidget         *widget,
             {
               if (insensitive && ((x + y) % 2))
                 {
-                  *p++ = insens[0];
-                  *p++ = insens[1];
-                  *p++ = insens[2];
+                  GIMP_CAIRO_RGB24_SET_PIXEL (p,
+                                              insens[0],
+                                              insens[1],
+                                              insens[2]);
                 }
               else
                 {
-                  *p++ = opaque[0];
-                  *p++ = opaque[1];
-                  *p++ = opaque[2];
+                  GIMP_CAIRO_RGB24_SET_PIXEL (p,
+                                              opaque[0],
+                                              opaque[1],
+                                              opaque[2]);
                 }
+
+              p += 4;
             }
         }
 
@@ -591,9 +615,11 @@ _gimp_color_area_render_buf (GtkWidget         *widget,
         {
           if ((width - x) * height > y * width)
             {
-              *p++ = opaque[0];
-              *p++ = opaque[1];
-              *p++ = opaque[2];
+              GIMP_CAIRO_RGB24_SET_PIXEL (p,
+                                          opaque[0],
+                                          opaque[1],
+                                          opaque[2]);
+              p += 4;
 
               continue;
             }
@@ -604,53 +630,60 @@ _gimp_color_area_render_buf (GtkWidget         *widget,
             {
               if ((gint) frac)
                 {
-                  *p++ = light[0];
-                  *p++ = light[1];
-                  *p++ = light[2];
+                  GIMP_CAIRO_RGB24_SET_PIXEL (p,
+                                              light[0],
+                                              light[1],
+                                              light[2]);
                 }
               else
                 {
-                  *p++ = ((gdouble) light[0]  * frac +
-                          (gdouble) opaque[0] * (1.0 - frac));
-                  *p++ = ((gdouble) light[1]  * frac +
-                          (gdouble) opaque[1] * (1.0 - frac));
-                  *p++ = ((gdouble) light[2]  * frac +
-                          (gdouble) opaque[2] * (1.0 - frac));
+                  GIMP_CAIRO_RGB24_SET_PIXEL (p,
+                                              ((gdouble) light[0]  * frac +
+                                               (gdouble) opaque[0] * (1.0 - frac)),
+                                              ((gdouble) light[1]  * frac +
+                                               (gdouble) opaque[1] * (1.0 - frac)),
+                                              ((gdouble) light[2]  * frac +
+                                               (gdouble) opaque[2] * (1.0 - frac)));
                 }
             }
           else
             {
               if ((gint) frac)
                 {
-                  *p++ = dark[0];
-                  *p++ = dark[1];
-                  *p++ = dark[2];
+                  GIMP_CAIRO_RGB24_SET_PIXEL (p,
+                                              dark[0],
+                                              dark[1],
+                                              dark[2]);
                 }
               else
                 {
-                  *p++ = ((gdouble) dark[0] * frac +
-                          (gdouble) opaque[0] * (1.0 - frac));
-                  *p++ = ((gdouble) dark[1] * frac +
-                          (gdouble) opaque[1] * (1.0 - frac));
-                  *p++ = ((gdouble) dark[2] * frac +
-                          (gdouble) opaque[2] * (1.0 - frac));
+                  GIMP_CAIRO_RGB24_SET_PIXEL (p,
+                                              ((gdouble) dark[0] * frac +
+                                               (gdouble) opaque[0] * (1.0 - frac)),
+                                              ((gdouble) dark[1] * frac +
+                                               (gdouble) opaque[1] * (1.0 - frac)),
+                                              ((gdouble) dark[2] * frac +
+                                               (gdouble) opaque[2] * (1.0 - frac)));
                 }
             }
+
+          p += 4;
         }
     }
 }
+
 static void
 gimp_color_area_render (GimpColorArea *area)
 {
   if (! area->buf)
     return;
 
-  _gimp_color_area_render_buf (GTK_WIDGET (area),
-                               ! GTK_WIDGET_IS_SENSITIVE (area),
-                               area->type,
-                               area->buf,
-                               area->width, area->height, area->rowstride,
-                               &area->color);
+  gimp_color_area_render_buf (GTK_WIDGET (area),
+                              ! gtk_widget_is_sensitive (GTK_WIDGET (area)),
+                              area->type,
+                              area->buf,
+                              area->width, area->height, area->rowstride,
+                              &area->color);
 
   area->needs_render = FALSE;
 }
@@ -713,20 +746,17 @@ gimp_color_area_drag_data_received (GtkWidget        *widget,
                                     guint             time)
 {
   GimpColorArea *area = GIMP_COLOR_AREA (widget);
+  const guint16 *vals;
   GimpRGB        color;
-  guint16       *vals;
 
-  if (selection_data->length < 0)
-    return;
-
-  if ((selection_data->format != 16) ||
-      (selection_data->length != 8))
+  if (gtk_selection_data_get_length (selection_data) != 8 ||
+      gtk_selection_data_get_format (selection_data) != 16)
     {
-      g_warning ("Received invalid color data");
+      g_warning ("%s: received invalid color data", G_STRFUNC);
       return;
     }
 
-  vals = (guint16 *)selection_data->data;
+  vals = (const guint16 *) gtk_selection_data_get_data (selection_data);
 
   gimp_rgba_set (&color,
                  (gdouble) vals[0] / 0xffff,
