@@ -3,9 +3,9 @@
  * Copyright (C) 1997 Eiichi Takamori
  * Copyright (C) 1996, 1997 Torsten Martinsen
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -30,6 +29,7 @@
 
 #define PLUG_IN_PROC    "plug-in-engrave"
 #define PLUG_IN_BINARY  "engrave"
+#define PLUG_IN_ROLE    "gimp-engrave"
 #define SCALE_WIDTH     125
 #define TILE_CACHE_SIZE  16
 
@@ -67,7 +67,7 @@ static void      engrave_small  (GimpDrawable *drawable,
 static void      engrave_sub    (gint          height,
                                  gboolean      limit,
                                  gint          bpp,
-                                 gint          color_n);
+                                 gint          num_channels);
 
 const GimpPlugInInfo PLUG_IN_INFO =
 {
@@ -90,11 +90,11 @@ query (void)
 {
   static const GimpParamDef args[] =
   {
-    { GIMP_PDB_INT32,    "run-mode", "Interactive, non-interactive" },
-    { GIMP_PDB_IMAGE,    "image",    "Input image (unused)"         },
-    { GIMP_PDB_DRAWABLE, "drawable", "Input drawable"               },
-    { GIMP_PDB_INT32,    "height",   "Resolution in pixels"         },
-    { GIMP_PDB_INT32,    "limit",    "If true, limit line width"    }
+    { GIMP_PDB_INT32,    "run-mode", "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }" },
+    { GIMP_PDB_IMAGE,    "image",    "Input image (unused)"             },
+    { GIMP_PDB_DRAWABLE, "drawable", "Input drawable"                   },
+    { GIMP_PDB_INT32,    "height",   "Resolution in pixels"             },
+    { GIMP_PDB_INT32,    "limit",    "Limit line width { TRUE, FALSE }" }
   };
 
   gimp_install_procedure (PLUG_IN_PROC,
@@ -206,7 +206,7 @@ engrave_dialog (GimpDrawable *drawable)
 
   gimp_ui_init (PLUG_IN_BINARY, FALSE);
 
-  dialog = gimp_dialog_new (_("Engrave"), PLUG_IN_BINARY,
+  dialog = gimp_dialog_new (_("Engrave"), PLUG_IN_ROLE,
                             NULL, 0,
                             gimp_standard_help_func, PLUG_IN_PROC,
 
@@ -222,9 +222,10 @@ engrave_dialog (GimpDrawable *drawable)
 
   gimp_window_set_transient (GTK_WINDOW (dialog));
 
-  main_vbox = gtk_vbox_new (FALSE, 12);
+  main_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
   gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 12);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), main_vbox);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+                      main_vbox, TRUE, TRUE, 0);
   gtk_widget_show (main_vbox);
 
   preview = gimp_drawable_preview_new (drawable, NULL);
@@ -424,7 +425,7 @@ engrave_small (GimpDrawable *drawable,
                GimpPreview  *preview)
 {
   GimpPixelRgn src_rgn, dest_rgn;
-  gint         bpp, color_n;
+  gint         bpp, num_channels;
   gint         x1, y1, x2, y2;
   gint         width, height;
   gint         progress, max_progress;
@@ -462,7 +463,7 @@ engrave_small (GimpDrawable *drawable,
   max_progress = width * height;
 
   bpp = drawable->bpp;
-  color_n = (gimp_drawable_is_rgb (drawable->drawable_id)) ? 3 : 1;
+  num_channels = (gimp_drawable_is_rgb (drawable->drawable_id)) ? 3 : 1;
 
   area.width = (tile_width / line_height) * line_height;
   area.data = g_new(guchar, (glong) bpp * area.width * area.width);
@@ -477,7 +478,7 @@ engrave_small (GimpDrawable *drawable,
           gimp_pixel_rgn_get_rect (&src_rgn, area.data, area.x, area.y, 1,
                                    area.h);
 
-          engrave_sub (line_height, limit, bpp, color_n);
+          engrave_sub (line_height, limit, bpp, num_channels);
 
           gimp_pixel_rgn_set_rect (&dest_rgn, area.data,
                                    area.x, area.y, 1, area.h);
@@ -500,6 +501,7 @@ engrave_small (GimpDrawable *drawable,
     }
   else
     {
+      gimp_progress_update (1.0);
       gimp_drawable_flush (drawable);
       gimp_drawable_merge_shadow (drawable->drawable_id, TRUE);
       gimp_drawable_update (drawable->drawable_id, x1, y1, x2 - x1, y2 - y1);
@@ -510,9 +512,9 @@ static void
 engrave_sub (gint height,
              gint limit,
              gint bpp,
-             gint color_n)
+             gint num_channels)
 {
-  glong average[3];             /* color_n <= 3 */
+  glong average[3];             /* num_channels <= 3 */
   gint y, h, inten, v;
   guchar *buf_row, *buf;
   gint row;
@@ -520,6 +522,8 @@ engrave_sub (gint height,
   gint count;
   gint i;
 
+  g_return_if_fail ((num_channels == 1) || (num_channels == 3));
+
   /*
     Since there's so many nested FOR's,
     put a few of them here...
@@ -532,7 +536,7 @@ engrave_sub (gint height,
       h = height - (y % height);
       h = MIN(h, area.y + area.h - y);
 
-      for (i = 0; i < color_n; i++)
+      for (i = 0; i < num_channels; i++)
         average[i] = 0;
       count = 0;
 
@@ -542,7 +546,7 @@ engrave_sub (gint height,
       for (row = 0; row < h; row++)
         {
           buf = buf_row;
-          for (i = 0; i < color_n; i++)
+          for (i = 0; i < num_channels; i++)
             average[i] += buf[i];
           count++;
           buf_row += rowstride;
@@ -550,10 +554,10 @@ engrave_sub (gint height,
 
       /* Average */
       if (count > 0)
-        for (i = 0; i < color_n; i++)
+        for (i = 0; i < num_channels; i++)
           average[i] /= count;
 
-      if (bpp < 3)
+      if (num_channels == 1)
         inten = average[0] / 254.0 * height;
       else
         inten = GIMP_RGB_LUMINANCE (average[0],
@@ -574,7 +578,7 @@ engrave_sub (gint height,
               else if (row == height-1)
                 v = 0;
             }
-          for (i = 0; i < color_n; i++)
+          for (i = 0; i < num_channels; i++)
             buf[i] = v;
           buf_row += rowstride;
         }
