@@ -316,7 +316,7 @@ static gint     GetCode      (FILE        *fd,
 static gint     LZWReadByte  (FILE        *fd,
                               gint         just_reset_LZW,
                               gint         input_code_size);
-static gint32   ReadImage    (FILE        *fd,
+static gboolean ReadImage    (FILE        *fd,
                               const gchar *filename,
                               gint         len,
                               gint         height,
@@ -328,7 +328,8 @@ static gint32   ReadImage    (FILE        *fd,
                               guint        leftpos,
                               guint        toppos,
                               guint        screenwidth,
-                              guint        screenheight);
+                              guint        screenheight,
+                              gint32      *image_ID);
 
 
 static gint32
@@ -346,6 +347,7 @@ load_image (const gchar  *filename,
   gint      imageCount = 0;
   gchar     version[4];
   gint32    image_ID = -1;
+  gboolean  status;
 
   fd = g_fopen (filename, "rb");
 
@@ -468,27 +470,34 @@ load_image (const gchar  *filename,
               return image_ID; /* will be -1 if failed on first image! */
             }
 
-          image_ID = ReadImage (fd, filename, LM_to_uint (buf[4], buf[5]),
-                                LM_to_uint (buf[6], buf[7]),
-                                localColorMap, bitPixel,
-                                grayScale,
-                                BitSet (buf[8], INTERLACE), imageCount,
-                                (guint) LM_to_uint (buf[0], buf[1]),
-                                (guint) LM_to_uint (buf[2], buf[3]),
-                                GifScreen.Width,
-                                GifScreen.Height);
+          status = ReadImage (fd, filename, LM_to_uint (buf[4], buf[5]),
+                              LM_to_uint (buf[6], buf[7]),
+                              localColorMap, bitPixel,
+                              grayScale,
+                              BitSet (buf[8], INTERLACE), imageCount,
+                              (guint) LM_to_uint (buf[0], buf[1]),
+                              (guint) LM_to_uint (buf[2], buf[3]),
+                              GifScreen.Width,
+                              GifScreen.Height,
+                              &image_ID);
         }
       else
         {
-          image_ID = ReadImage (fd, filename, LM_to_uint (buf[4], buf[5]),
-                                LM_to_uint (buf[6], buf[7]),
-                                GifScreen.ColorMap, GifScreen.BitPixel,
-                                GifScreen.GrayScale,
-                                BitSet (buf[8], INTERLACE), imageCount,
-                                (guint) LM_to_uint (buf[0], buf[1]),
-                                (guint) LM_to_uint (buf[2], buf[3]),
-                                GifScreen.Width,
-                                GifScreen.Height);
+          status = ReadImage (fd, filename, LM_to_uint (buf[4], buf[5]),
+                              LM_to_uint (buf[6], buf[7]),
+                              GifScreen.ColorMap, GifScreen.BitPixel,
+                              GifScreen.GrayScale,
+                              BitSet (buf[8], INTERLACE), imageCount,
+                              (guint) LM_to_uint (buf[0], buf[1]),
+                              (guint) LM_to_uint (buf[2], buf[3]),
+                              GifScreen.Width,
+                              GifScreen.Height,
+                              &image_ID);
+        }
+
+      if (!status)
+        {
+          break;
         }
 
       if (comment_parasite != NULL)
@@ -655,7 +664,7 @@ GetDataBlock (FILE   *fd,
       return -1;
     }
 
-  ZeroDataBlock = count == 0;
+  ZeroDataBlock = (count == 0);
 
   if ((count != 0) && (! ReadOK (fd, buf, count)))
     {
@@ -697,8 +706,15 @@ GetCode (FILE     *fd,
       buf[0] = buf[last_byte - 2];
       buf[1] = buf[last_byte - 1];
 
-      if ((count = GetDataBlock (fd, &buf[2])) <= 0)
-        done = TRUE;
+      count = GetDataBlock (fd, &buf[2]);
+      if (count < 0)
+        {
+          return -1;
+        }
+      else if (count == 0)
+        {
+          done = TRUE;
+        }
 
       last_byte = 2 + count;
       curbit = (curbit - lastbit) + 16;
@@ -732,7 +748,7 @@ LZWReadByte (FILE *fd,
 
   if (just_reset_LZW)
     {
-      if (input_code_size > MAX_LZW_BITS)
+      if (input_code_size > MAX_LZW_BITS || input_code_size <= 1)
         {
           g_message ("Value out of range for code size (corrupted file?)");
           return -1;
@@ -745,7 +761,10 @@ LZWReadByte (FILE *fd,
       max_code_size = 2 * clear_code;
       max_code      = clear_code + 2;
 
-      GetCode (fd, 0, TRUE);
+      if (GetCode (fd, 0, TRUE) < 0)
+        {
+          return -1;
+        }
 
       fresh = TRUE;
 
@@ -773,6 +792,11 @@ LZWReadByte (FILE *fd,
         }
       while (firstcode == clear_code);
 
+      if (firstcode < 0)
+        {
+          return -1;
+        }
+
       return firstcode & 255;
     }
 
@@ -800,6 +824,11 @@ LZWReadByte (FILE *fd,
           sp            = stack;
           firstcode     = oldcode = GetCode (fd, code_size, FALSE);
 
+          if (firstcode < 0)
+            {
+              return -1;
+            }
+
           return firstcode & 255;
         }
       else if (code == end_code || code > max_code)
@@ -861,10 +890,15 @@ LZWReadByte (FILE *fd,
         return (*--sp) & 255;
     }
 
+  if (code < 0)
+    {
+      return -1;
+    }
+
   return code & 255;
 }
 
-static gint32
+static gboolean
 ReadImage (FILE        *fd,
            const gchar *filename,
            gint         len,
@@ -877,9 +911,9 @@ ReadImage (FILE        *fd,
            guint        leftpos,
            guint        toppos,
            guint        screenwidth,
-           guint        screenheight)
+           guint        screenheight,
+           gint32      *image_ID)
 {
-  static gint32 image_ID   = -1;
   static gint   frame_number = 1;
 
   gint32        layer_ID;
@@ -900,7 +934,8 @@ ReadImage (FILE        *fd,
   if (len < 1 || height < 1)
     {
       g_message ("Bogus frame dimensions");
-      return -1;
+      *image_ID = -1;
+      return FALSE;
     }
 
   /*
@@ -909,13 +944,15 @@ ReadImage (FILE        *fd,
   if (! ReadOK (fd, &c, 1))
     {
       g_message ("EOF / read error on image data");
-      return -1;
+      *image_ID = -1;
+      return FALSE;
     }
 
   if (LZWReadByte (fd, TRUE, c) < 0)
     {
       g_message ("Error while reading");
-      return -1;
+      *image_ID = -1;
+      return FALSE;
     }
 
   if (frame_number == 1)
@@ -927,8 +964,8 @@ ReadImage (FILE        *fd,
       if (screenheight == 0)
         screenheight = height;
 
-      image_ID = gimp_image_new (screenwidth, screenheight, GIMP_INDEXED);
-      gimp_image_set_filename (image_ID, filename);
+      *image_ID = gimp_image_new (screenwidth, screenheight, GIMP_INDEXED);
+      gimp_image_set_filename (*image_ID, filename);
 
       for (i = 0, j = 0; i < ncols; i++)
         {
@@ -937,7 +974,7 @@ ReadImage (FILE        *fd,
           used_cmap[2][i] = gimp_cmap[j++] = cmap[2][i];
         }
 
-      gimp_image_set_colormap (image_ID, gimp_cmap, ncols);
+      gimp_image_set_colormap (*image_ID, gimp_cmap, ncols);
 
       if (Gif89.delayTime < 0)
         framename = g_strdup (_("Background"));
@@ -949,13 +986,13 @@ ReadImage (FILE        *fd,
 
       if (Gif89.transparent == -1)
         {
-          layer_ID = gimp_layer_new (image_ID, framename,
+          layer_ID = gimp_layer_new (*image_ID, framename,
                                      len, height,
                                      GIMP_INDEXED_IMAGE, 100, GIMP_NORMAL_MODE);
         }
       else
         {
-          layer_ID = gimp_layer_new (image_ID, framename,
+          layer_ID = gimp_layer_new (*image_ID, framename,
                                      len, height,
                                      GIMP_INDEXEDA_IMAGE, 100, GIMP_NORMAL_MODE);
           alpha_frame=TRUE;
@@ -986,7 +1023,7 @@ ReadImage (FILE        *fd,
 #ifdef GIFDEBUG
                   g_print ("GIF: Promoting image to RGB...\n");
 #endif
-                  gimp_image_convert_rgb (image_ID);
+                  gimp_image_convert_rgb (*image_ID);
 
                   break;
                 }
@@ -1038,7 +1075,7 @@ ReadImage (FILE        *fd,
         }
       previous_disposal = Gif89.disposal;
 
-      layer_ID = gimp_layer_new (image_ID, framename,
+      layer_ID = gimp_layer_new (*image_ID, framename,
                                  len, height,
                                  promote_to_rgb ?
                                  GIMP_RGBA_IMAGE : GIMP_INDEXEDA_IMAGE,
@@ -1049,7 +1086,7 @@ ReadImage (FILE        *fd,
 
   frame_number++;
 
-  gimp_image_insert_layer (image_ID, layer_ID, -1, 0);
+  gimp_image_insert_layer (*image_ID, layer_ID, -1, 0);
   gimp_layer_translate (layer_ID, (gint) leftpos, (gint) toppos);
 
   drawable = gimp_drawable_get (layer_ID);
@@ -1061,7 +1098,8 @@ ReadImage (FILE        *fd,
   {
     g_message ("'%s' has a larger image size than GIMP can handle.",
                gimp_filename_to_utf8 (filename));
-    return -1;
+    *image_ID = -1;
+    return FALSE;
   }
 
   if (alpha_frame)
@@ -1171,10 +1209,12 @@ ReadImage (FILE        *fd,
         break;
     }
 
- fini:
-  if (LZWReadByte (fd, FALSE, c) >= 0)
-    g_print ("GIF: too much input data, ignoring extra...\n");
+  if (v < 0)
+    {
+      return FALSE;
+    }
 
+ fini:
   gimp_progress_update (1.0);
   gimp_pixel_rgn_init (&pixel_rgn, drawable,
                        0, 0, drawable->width, drawable->height, TRUE, FALSE);
@@ -1186,5 +1226,11 @@ ReadImage (FILE        *fd,
   gimp_drawable_flush (drawable);
   gimp_drawable_detach (drawable);
 
-  return image_ID;
+  if (LZWReadByte (fd, FALSE, c) >= 0)
+    {
+      g_print ("GIF: too much input data, ignoring extra...\n");
+      return FALSE;
+    }
+
+  return TRUE;
 }
