@@ -72,7 +72,9 @@ static  void      write_image     (FILE   *f,
                                    gint    bpp,
                                    gint    spzeile,
                                    gint    MapSize,
-                                   RGBMode rgb_format);
+                                   RGBMode rgb_format,
+                                   gint    mask_info_size,
+                                   gint    color_space_size);
 
 static  gboolean  save_dialog     (gint    channels);
 
@@ -81,18 +83,18 @@ static void
 FromL (gint32  wert,
        guchar *bopuffer)
 {
-  bopuffer[0] = (wert & 0x000000ff)>>0x00;
-  bopuffer[1] = (wert & 0x0000ff00)>>0x08;
-  bopuffer[2] = (wert & 0x00ff0000)>>0x10;
-  bopuffer[3] = (wert & 0xff000000)>>0x18;
+  bopuffer[0] = (wert)         & 0xff;
+  bopuffer[1] = (wert >> 0x08) & 0xff;
+  bopuffer[2] = (wert >> 0x10) & 0xff;
+  bopuffer[3] = (wert >> 0x18) & 0xff;
 }
 
 static void
 FromS (gint16  wert,
        guchar *bopuffer)
 {
-  bopuffer[0] = (wert & 0x00ff)>>0x00;
-  bopuffer[1] = (wert & 0xff00)>>0x08;
+  bopuffer[0] = (wert)         & 0xff;
+  bopuffer[1] = (wert >> 0x08) & 0xff;
 }
 
 static void
@@ -268,6 +270,7 @@ WriteBMP (const gchar  *filename,
       if (interactive && !save_dialog (channels))
         return GIMP_PDB_CANCEL;
 
+      /* mask_info_size is only set to non-zero for 16- and 32-bpp */
       switch (BMPSaveData.rgb_format)
         {
         case RGB_888:
@@ -275,6 +278,7 @@ WriteBMP (const gchar  *filename,
           break;
         case RGBA_8888:
           BitsPerPixel = 32;
+          mask_info_size = 16;
           break;
         case RGBX_8888:
           BitsPerPixel = 32;
@@ -290,6 +294,7 @@ WriteBMP (const gchar  *filename,
           break;
         case RGB_555:
           BitsPerPixel = 16;
+          mask_info_size = 16;
           break;
         default:
           g_return_val_if_reached (GIMP_PDB_EXECUTION_ERROR);
@@ -335,9 +340,10 @@ WriteBMP (const gchar  *filename,
   else
     SpZeile = ((gint) (((Spcols * BitsPerPixel) / 8) / 4) + 1) * 4;
 
-  color_space_size = 0;
   if (! BMPSaveData.dont_write_color_space_data)
     color_space_size = 68;
+  else
+    color_space_size = 0;
 
   Bitmap_File_Head.bfSize    = (0x36 + MapSize + (rows * SpZeile) +
                                 mask_info_size + color_space_size);
@@ -522,7 +528,8 @@ WriteBMP (const gchar  *filename,
                pixels, cols, rows,
                BMPSaveData.use_run_length_encoding,
                channels, BitsPerPixel, SpZeile,
-               MapSize, BMPSaveData.rgb_format);
+               MapSize, BMPSaveData.rgb_format,
+               mask_info_size, color_space_size);
 
   /* ... and exit normally */
 
@@ -564,7 +571,9 @@ write_image (FILE   *f,
              gint    bpp,
              gint    spzeile,
              gint    MapSize,
-             RGBMode rgb_format)
+             RGBMode rgb_format,
+             gint    mask_info_size,
+             gint    color_space_size)
 {
   guchar  buf[16] = { 0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0 };
   guchar  puffer[8];
@@ -599,20 +608,20 @@ write_image (FILE   *f,
                   Write (f, buf, 3);
                   break;
                 case RGBX_8888:
-                  buf[0] = 0;
                   buf[3] = *temp++;
                   buf[2] = *temp++;
                   buf[1] = *temp++;
+                  buf[0] = 0;
                   xpos++;
                   if (channels > 3 && (guchar) *temp == 0)
                     buf[0] = buf[1] = buf[2] = 0xff;
                   Write (f, buf, 4);
                   break;
                 case RGBA_8888:
+                  buf[3] = *temp++;
                   buf[2] = *temp++;
                   buf[1] = *temp++;
-                  buf[0] = *temp++;
-                  buf[3] = *temp;
+                  buf[0] = *temp;
                   xpos++;
                   Write (f, buf, 4);
                   break;
@@ -812,7 +821,7 @@ write_image (FILE   *f,
             FromL (length, puffer);
             Write (f, puffer, 4);
             fseek (f, 0x02, SEEK_SET);            /* Write length of file */
-            length += (0x36 + MapSize);
+            length += (0x36 + MapSize + mask_info_size + color_space_size);
             FromL (length, puffer);
             Write (f, puffer, 4);
             g_free (ketten);
